@@ -29,6 +29,9 @@ class CanonicalException(Exception):
     #: The default log message. String formatting is applied.
     log_message: str = "Caught fatal {code} (id: {id})"
 
+    #: The cooldown period enforced on the caller.
+    backoff: dict = None
+
     @property
     def http_headers(self) -> dict:
         """The HTTP headers when returning this exception to a client."""
@@ -64,6 +67,21 @@ class CanonicalException(Exception):
         """Use `logger` to log the exception as `level`."""
         return func((message or self.log_message).format(**self.as_dict()))
 
+    def set_backoff(self,
+        ttl: int,
+        expires: int,
+        attempts: int
+    ) -> None:
+        """Set a backoff (cooldown) period that indicating how
+        long the caller wait before retrying the request that
+        caused the :class:`CanonicalException`.
+        """
+        self.backoff = {
+            'ttl': ttl,
+            'expires': expires,
+            'attempts': attempts
+        }
+
     def __str__(self) -> str:
         return repr(self)
 
